<?php
/*
 * Plugin Name:       Docage
 * Plugin URI:        https://docage.com
 * Description:       Fill Docage templates with WPForms entries.
 * Version:           1.2.5
 * Author:            Docage
 * Author URI:        https://docage.com
 * Text Domain:       docage */

include_once("classes/Binding.php");
include_once("classes/DocageTemplate.php");
include_once("classes/DocageTemplateMember.php");
include_once("classes/DocageField.php");
include_once("classes/WPForm.php");
include_once("classes/WPFormField.php");

include_once("functions.php");
include_once("db_manager.php");
include_once("connection_manager.php");
include_once("api_manager.php");
include_once("get_data_actions.php");

session_start();

register_activation_hook(__FILE__, 'createOrUpdateDb');

function docage_admin_menu_function()
{
    add_menu_page('Forms match', 'Docage', 'manage_options', 'docage-admin-menu', 'docage_plugin_page', '', 10);
}

add_action('admin_menu', 'docage_admin_menu_function');

function docage_plugin_page()
{
    // Check if logout button has been clicked to clear the session
    if (array_key_exists("logout_submit", $_POST)) {
        $_SESSION = array();
    }

    // Session variables declaration
    // Language choice
    if (array_key_exists("language_choice_fr", $_POST)) $_SESSION['lang'] = "FR";
    else if (array_key_exists("language_choice_en", $_POST)) $_SESSION['lang'] = "EN";
    else if (array_key_exists("language_choice_es", $_POST)) $_SESSION['lang'] = "ES";

    // The default language is French
    if (!isset($_SESSION['lang'])) $_SESSION['lang'] = "FR";

    if (!isset($_SESSION['is_authenticated'])) $_SESSION['is_authenticated'] = false;


    // Check if the authentication button has been clicked to check if credentials are correct
    if (array_key_exists("authentication_submit", $_POST)) {
        checkAndSaveCredentials($_POST["login_username"], $_POST["login_api_key"]);
    }

    // Check if correct informations are still correct and get loggged user informations
    if ($_SESSION['is_authenticated'] == true) {
        $response = Docage_API_request("GET", "DocageUsers/ByEmail/".$_SESSION['DocageApiUser']);
        if ($response[0] == 200)
            $_SESSION['loggedUserInformations'] = $response[1];
        else
            $_SESSION['is_authenticated'] = false;
    }

    // If user is logged in
    if ($_SESSION['is_authenticated'] == true) {

        // Find all data lists and place them in session, to use them in JS
        // Find Docage templates from the authenticated user
        $_SESSION['transaction_templates'] = getAllDocageTemplates();

        // Find Transaction and TransactionMember properties
        $_SESSION['transactionProperties'] = getTransactionProperties();
        $_SESSION['transactionMemberProperties'] = getTransactionMemberProperties();
        /*$_SESSION['transactionFileProperties'] = getTransactionFileProperties();*/

        // Find custom fields of user's organization
        $allCustomFields = getAllCustomFields($_SESSION['loggedUserInformations']->OrganizationId);
        $_SESSION['transactionCustomFields'] = $allCustomFields->Transaction;
        $_SESSION['contactCustomFields'] = $allCustomFields->Contact;

        // Find WPForms from the WP database
        $_SESSION['wpforms_array'] = getAllWpForms();

        // Find existing bindings from the WP database
        $_SESSION['bindings_array'] = getAllBindings();
    }

    ?>

    <!-- Plugin interface -->
    <style type="text/css">
        @import "<?= plugins_url( 'css/docage_style.css', __FILE__ ) ?>";
        @import "https://cdn.jsdelivr.net/npm/toastify-js/src/toastify.min.css";

        /* Style classes that require PHP */
        @font-face {
            font-family: "Roboto";
            src: url("<?= plugins_url( 'fonts/Roboto/Roboto-Regular.ttf', __FILE__ ) ?>");
        }

        #flag_fr {
            background-image: url("<?= plugins_url( 'pages/img/flag_fr.jpg', __FILE__ ) ?>");
        }

        #flag_en {
            background-image: url("<?= plugins_url( 'pages/img/flag_en.jpg', __FILE__ ) ?>");
        }

        #flag_es {
            background-image: url("<?= plugins_url( 'pages/img/flag_es.jpg', __FILE__ ) ?>");
        }

        #home_container,
        #transactions_container {
            width: 95%;
            padding: 30px;
            border: 1px solid #f1f1f1;
            background: #fff;
            box-shadow: 0 0 20px 0 rgba(0, 0, 0, 0.2), 0 5px 5px 0 rgba(0, 0, 0, 0.24);
            /*background-image: url("<?= plugins_url( 'pages/img/filigrane.jpg', __FILE__ ) ?>");*/
            background-position: bottom;
            text-align: left;
            vertical-align: top;
        }

        #infos_container .container {
            width: 1000px;
            margin: auto;
            margin-top: 20px;
            margin-bottom: 20px;
            padding: 40px;

            background-color: white;
            border-color: #849460;
            border-style: solid;
            border-width: 2px;
            border-radius: 20px;

            text-align: justify;

            background-image: url('<?= plugins_url( 'pages/img/filigrane.jpg', __FILE__ ) ?>');
            background-position: bottom left;
            background-repeat: no-repeat;
        }
    </style>

    <div class="docage-wrapper">
        <div class="docage-header">
            <img id='docage_logo' src="<?= plugins_url('pages/img/docage_logo.png', __FILE__) ?>">
        </div>
        <div id="menu">
            <ul class='menu'>
                <?php
                if ($_SESSION['lang'] === 'FR') echo "<li id='home_tab_display'><a href=''>Paramètres</a></li>";
                else if ($_SESSION['lang'] === 'EN') echo "<li id='home_tab_display'><a href=''>Settings</a></li>";
                else if ($_SESSION['lang'] === 'ES') echo "<li id='home_tab_display'><a href=''>Ajustes</a></li>";

                if ($_SESSION['lang'] === 'FR') echo "<li id='transactions_tab_display'><a href=''>Transactions</a></li>";
                else if ($_SESSION['lang'] === 'EN') echo "<li id='transactions_tab_display'><a href=''>Transactions</a></li>";
                else if ($_SESSION['lang'] === 'ES') echo "<li id='transactions_tab_display'><a href=''>Transactions</a></li>";

                if ($_SESSION['lang'] === 'FR') echo "<li id='infos_tab_display'><a href=''>Aide</a></li>";
                else if ($_SESSION['lang'] === 'EN') echo "<li id='infos_tab_display'><a href=''>Help</a></li>";
                else if ($_SESSION['lang'] === 'ES') echo "<li id='infos_tab_display'><a href=''>Ayuda</a></li>";
                ?>
            </ul>
        </div>
        <div id="infos_container" style="display: none;">
            <?php include 'pages/plugin_infos_tab.php' ?>
        </div>
        <div id="transactions_container" <?php if (!isset($_GET['currentPage'])) { ?> style="display: none" <?php } ?>>
            <?php if ($_SESSION['is_authenticated'] === true) { ?>
                <?php include 'pages/transactions_list_tab.php' ?>
            <?php } else {
                echo 'Veuillez vous connecter';
            } ?>
        </div>

        <div id='home_container' <?php if (isset($_GET['currentPage'])) { ?> style="display: none" <?php } ?>>
            <?php include 'pages/home_tab.php' ?>
        </div>
    </div>

    <?php include("scripts/docage_script.php") ?>

<?php }

// Wordpress part, when bindings are created
function url_to_path( $url ) {
    return str_replace(
        wp_get_upload_dir()['baseurl'],
        wp_get_upload_dir()['basedir'],
        $url
    );
}

function docage_process_form($formFields, $userEntry, $formInfos) {
    global $wpdb;

    // Get all bindings associated with this form
    $bindingsFromDb = Database_connect("SELECT", "SELECT * FROM ".$wpdb->prefix."docage_bindings JOIN ".$wpdb->prefix."docage_credentials ON ".$wpdb->prefix."docage_bindings.CredentialsId = ".$wpdb->prefix."docage_credentials.id WHERE
		FormProvider	= 'WPForms' AND
		TemplateType	= 'Transaction' AND
		FormId			= '$formInfos[id]'
		")->fetchAll();

    $docageFormDataCreation["TemplateId"] = $bindingsFromDb[0]['TemplateId'];

    foreach ($bindingsFromDb as $bindingFromDb) 
    {
        foreach ($formFields as $formField) 
        {
            if (str_replace(" ", "-", $formField['name']) == $bindingFromDb['FormFieldName']) 
            {
                if (strpos($bindingFromDb['TemplateObjectProperty'], "custom-") === 0) 
                {
                    $customField = new stdClass();
                    $customField->Type = $bindingFromDb['FormFieldType'];
                    switch ($bindingFromDb['FormFieldType']) 
                    {
                        case "radio": $customField->Type = "radio-group"; break;
                        case "checkbox": $customField->Type = "checkbox-group"; break;
                        // case "date-time": $customField->Type = "date"; break;
                        // case "email": $customField->Type = "text"; break;
                        // case "phone": $customField->Type = "text"; break;
                        // case "password": $customField->Type = "text"; break;
                        // case "number-slider": $customField->Type = "number"; break;
                        // case "payment-single": $customField->Type = "number"; break;
                        case "payment-multiple": $customField->Type = "radio-group"; break;
                        case "payment-checkbox": $customField->Type = "checkbox-group"; break;
                        case "payment-select": $customField->Type = "select"; break;
                        //case "payment-total": $customField->Type = "number"; break;
                        // case "rating": $customField->Type = "number"; break;
                        // case "url": $customField->Type = "text"; break;
                        // case "hidden": $customField->Type = "text"; break;
                        default: $customField->Type = $bindingFromDb['FormFieldType']; break;
                    }
                    $customField->Name = str_replace("custom-", "", $bindingFromDb['TemplateObjectProperty']);
                    if ($customField->Type == "select" || $customField->Type == "checkbox-group" || $customField->Type == "radio-group")
                        $customField->UserData = json_encode(explode("\n", $formField['value']));
                    else
                        $customField->UserData = $formField['value'];

                    error_log(json_encode($formField['value']));
                    if($bindingFromDb['TemplateObjectType'] != "TransactionMember")
                        $key = $bindingFromDb['TemplateObjectType'] . "[" . $bindingFromDb['TemplateObjectName'] . "]_" . $customField->Name;
                    else
                        $key = $bindingFromDb['TemplateObjectType'] . "[" . $bindingFromDb['TemplateObjectName'] . "]_Contact_" . $customField->Name;
                    $docageFormDataUpdate[$key] = $customField->UserData;
                } 
                else 
                {
                    if ($bindingFromDb['TemplateObjectType'] == "TransactionMember") 
                    {
                        $key = "TransactionMember[" . $bindingFromDb['TemplateObjectName'] . "]_Contact_" . $bindingFromDb['TemplateObjectProperty'];
                        $docageFormDataUpdate[$key] = $formField['value'];
                    } 
                    elseif($bindingFromDb['TemplateObjectType'] == "TransactionFile") 
                    {
                        $cFile = url_to_path($formField['value']);
                        $key = "TransactionFile[" . $bindingFromDb['TemplateObjectName'] . "]_File";
                        $docageFormDataUpdate[$key] = curl_file_create($cFile, "application/pdf", basename($cFile));
                    } 
                    else 
                    {
                        $key = "Transaction_" . $bindingFromDb['TemplateObjectProperty'];
                        $docageFormDataUpdate[$key] = $formField['value'];
                    }
                }
            }
        }
    }

    $curlCreation = curl_init();

    curl_setopt_array($curlCreation, array(
        CURLOPT_URL => "https://api.docage.com/Transactions/CreateFullTransaction?launchTransaction=false",
        CURLOPT_RETURNTRANSFER => true,
        CURLOPT_ENCODING => "",
        CURLOPT_MAXREDIRS => 10,
        CURLOPT_TIMEOUT => 0,
        CURLOPT_FOLLOWLOCATION => true,
        CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
        CURLOPT_CUSTOMREQUEST => "POST",
        CURLOPT_POSTFIELDS => $docageFormDataCreation,
        CURLOPT_USERPWD => $bindingsFromDb[0]['DocageUsername'] . ':' . $bindingsFromDb[0]['DocageApiKey']
    ));

    $responseCreation = curl_exec($curlCreation);
    $responseCodeCreation = curl_getinfo($curlCreation, CURLINFO_HTTP_CODE);
    if(!$responseCodeCreation == 200) {
        console_log("CURL Error when creating transaction: API return $responseCodeCreation " . curl_error($curlCreation));
        error_log("CURL Error when creating transaction: API return $responseCodeCreation " . curl_error($curlCreation));
        return $responseCodeCreation;
    }
        

    curl_close($curlCreation);

    $decoded_response = json_decode($responseCreation, true);

    $curlUpdate = curl_init();

    curl_setopt_array($curlUpdate, array(
        CURLOPT_URL => "https://api.docage.com/Transactions/UpdateTransactionFromForm/" . $decoded_response["Id"],
        CURLOPT_RETURNTRANSFER => true,
        CURLOPT_ENCODING => "",
        CURLOPT_MAXREDIRS => 10,
        CURLOPT_TIMEOUT => 0,
        CURLOPT_FOLLOWLOCATION => true,
        CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
        CURLOPT_CUSTOMREQUEST => "PUT",
        CURLOPT_POSTFIELDS => $docageFormDataUpdate,
        CURLOPT_USERPWD => $bindingsFromDb[0]['DocageUsername'] . ':' . $bindingsFromDb[0]['DocageApiKey']
    ));

    $responseUpdate = curl_exec($curlUpdate);
    $responseCodeUpdate = curl_getinfo($curlUpdate, CURLINFO_HTTP_CODE);
    if(!$responseCodeUpdate == 200) {
        console_log("CURL Error when updating transaction: API return $responseCodeUpdate " . curl_error($curlUpdate));
        error_log("CURL Error when updating transaction: API return $responseCodeUpdate " . curl_error($curlUpdate));
        return $responseCodeUpdate;
    }

    curl_close($curlUpdate);

    $decoded_responseUpdate = json_decode($responseUpdate, true);

    return $responseCodeUpdate;
}

// // When a form is submitted by the user
// function docage_process_form($formFields, $userEntry, $formInfos) {
//     global $wpdb;

//     console_log("ici");

//     // Get all bindings associated with this form
//     $bindingsFromDb = Database_connect("SELECT", "SELECT * FROM ".$wpdb->prefix."docage_bindings JOIN ".$wpdb->prefix."docage_credentials ON ".$wpdb->prefix."docage_bindings.CredentialsId = ".$wpdb->prefix."docage_credentials.id WHERE
// 		FormProvider	= 'WPForms' AND
// 		TemplateType	= 'Transaction' AND
// 		FormId			= '$formInfos[id]'
// 		")->fetchAll();

//     $docageFormData["TemplateId"] = $bindingsFromDb[0]['TemplateId'];

//     // For all bindings, find corresponding field from form
//     foreach ($bindingsFromDb as $bindingFromDb) {
//         foreach ($formFields as $formField) {
//             if (str_replace(" ", "-", $formField['name']) == $bindingFromDb['FormFieldName']) {
//                 if (strpos($bindingFromDb['TemplateObjectProperty'], "custom-") === 0) {
//                     $customField = new stdClass();
//                     switch ($bindingFromDb['FormFieldType']) {
//                         case "radio": $customField->Type = "radio-group"; break;
//                         case "date-time": $customField->Type = "date"; break;
//                         case "email": $customField->Type = "text"; break;
//                         case "phone": $customField->Type = "text"; break;
//                         case "password": $customField->Type = "text"; break;
//                         case "number-slider": $customField->Type = "number"; break;
//                         case "payment-single": $customField->Type = "number"; break;
//                         case "payment-multiple": $customField->Type = "radio-group"; break;
//                         case "payment-checkbox": $customField->Type = "checkbox-group"; break;
//                         case "payment-select": $customField->Type = "select"; break;
//                         case "payment-total": $customField->Type = "number"; break;
//                         case "rating": $customField->Type = "number"; break;
//                         case "url": $customField->Type = "text"; break;
//                         case "hidden": $customField->Type = "text"; break;
//                         default: $customField->Type = $bindingFromDb['FormFieldType']; break;
//                     }
//                     $customField->Name = str_replace("custom-", "", $bindingFromDb['TemplateObjectProperty']);
//                     $customField->UserData = [$formField['value']];
//                     // $docageFormData[$bindingFromDb['TemplateObjectName']]['CustomFieldsJson'][] = $customField;
//                     $docageFormData[$bindingFromDb['TemplateObjectName']][$customField->Name] = $customField->UserData;
//                 } else {
//                     if ($bindingFromDb['TemplateObjectType'] == "TransactionMember") {
//                         $docageFormData[$bindingFromDb['TemplateObjectName']][$bindingFromDb['TemplateObjectProperty']] = $formField['value'];
//                     } elseif($bindingFromDb['TemplateObjectType'] == "TransactionFile") {
//                         $cFile = url_to_path($formField['value']);
//                         $docageFormData[$bindingFromDb['TemplateObjectName']] = curl_file_create($cFile, "application/pdf", basename($cFile));
//                     } else {
//                         $docageFormData['Transaction'][$bindingFromDb['TemplateObjectProperty']] = $formField['value'];
//                     }
//                 }
//             }
//         }
//     }

//     // Encode in JSON the fields filled for a transaction member
//     foreach ($docageFormData as $friendlyName => $fields) {
//         $friendlyNameType = "";
//         foreach ($bindingsFromDb as $bindingFromDb) {
//             if ($bindingFromDb['TemplateObjectName'] == $friendlyName)
//                 $friendlyNameType = $bindingFromDb['TemplateObjectType'];
//         }
//         if($friendlyName != "TemplateId" && $friendlyNameType != "TransactionFile") {
//             if (isset($docageFormData[$friendlyName]["CustomFieldsJson"]))
//                 $docageFormData[$friendlyName]["CustomFieldsJson"] = json_encode($docageFormData[$friendlyName]["CustomFieldsJson"]);
//             if ($friendlyNameType == "TransactionMember")
//                 $docageFormData[$friendlyName] = json_encode($docageFormData[$friendlyName]);
//         }
//     }

//     $curl = curl_init();

//     curl_setopt_array($curl, array(
//         // CURLOPT_URL => "https://api.docage.com/Transactions/CreateFullTransaction",
//         CURLOPT_URL => "http://localhost:5000/Transactions/CreateFullTransaction",
//         CURLOPT_RETURNTRANSFER => true,
//         CURLOPT_ENCODING => "",
//         CURLOPT_MAXREDIRS => 10,
//         CURLOPT_TIMEOUT => 0,
//         CURLOPT_FOLLOWLOCATION => true,
//         CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
//         CURLOPT_CUSTOMREQUEST => "POST",
//         CURLOPT_POSTFIELDS => $docageFormData,
//         CURLOPT_USERPWD => $bindingsFromDb[0]['DocageUsername'] . ':' . $bindingsFromDb[0]['DocageApiKey']
//     ));

//     $response = curl_exec($curl);
//     if(!$response)
//         console_log("CURL Error: " . curl_error($curl));

//     curl_close($curl);

//     return $response;
// }

add_action('wpforms_process_filter', 'docage_process_form', 10, 3);

add_filter('plugin_action_links_'.plugin_basename(__FILE__), 'add_plugin_page_settings_link');

function add_plugin_page_settings_link( $links ) {
    array_unshift($links, '<a href="' .
        admin_url( 'options-general.php?page=docage-admin-menu' ) .
        '">' . __('Settings') . '</a>');

    return $links;
}
